{$B-}
{DYNAMIC DOCUMENT PACKAGE}

const                   { TO NOT INTERFERE WITH MEMORY NEEDED BY THE }
  chunksize = 8;        { PRINT ROUTINE, ADD (24*(MAXLEN+1) + 11)/16 }
  maxlen    = 135;                    { 205 WITH 135 }
  MinMem    = 275;      { ((maxlen + 1 * chunksize) + 8)/16 = MinMem }
  Invert    = 65536.0;                 {ROUND UP}
                               { EXAMPLE : WITH CHUNKSIZE = 8 }
                          { maxlen = 255 then MinMem = 128.5 = 129 }
type                      { maxlen = 230 then MinMem = 116.0 = 116 }
  natural = 0..maxint;    { maxlen = 135 then MinMem = 68.5  = 69  }
  cardinal = 1..maxint;           { DO NOT USE MAXLEN < 135 }
  stype = string[maxlen];
  short = string[80];
  ptr = ^chunk;
  chunk = record
            next : ptr;
            line : array [1..chunksize] of stype
          end;
  document = record
                w : stype;
                length : natural;
                position : 0..chunksize;
                start : ptr;
                current : ptr;
                chunkno : natural;
                status : (reading,writing,notready)
              end;
  relation = (before,beforeorequalto,equalto,afterorequalto,after,notequalto);

var
  test : char;
  a,b,c,d,e : document;
  ch,app : stype;
  locate : natural;
  availd : ptr;
  VideoBuffer,
  VideoPage : integer;
  left : natural;

{END OF GLOBAL DECLARATIONS -- LIBRARY PROCEDURES FOLLOW}


procedure dcreate (var d : document);   { INITIALIZES A DYNAMIC DOCUMENT }
forward;

procedure documenterror (n : natural);
begin
  writeln;
  writeln (' **** EXECUTION ERROR IN DOCUMENT LIBRARY  ****');
  case n of
       1 : write (' PUT ATTEMPTED IN READ STATE ');
       2 : write (' GET ATTEMPTED IN WRITE STATE ');
       3 : write (' GET ATTEMPTED BEYOND END OF DOCUMENT ');
       4 : write (' DELETE PORTION BIGGER THAN DOCUMENT ');
       5 : write (' EXTRACT PORTION BIGGER THAN DOCUMENT ');
       6 : write (' INSERTING BEYOND END OF DOCUMENT ');
  end;
  writeln (' ****');
  HALT;
end;

procedure newd (var p : ptr; var fail : boolean);
var
  I : 1..chunksize;              { MAKES A NEW CHUNK -- DOES NOT }
  test : real;                    { ACTUALLY ADD IT TO DOCUMENT }

begin
  if MemAvail < 0
    then
      test := Invert
    else
      test := 0.0;
  fail := false;
  if availd = nil     { IS THERE AN OLD CHUNK FLOATING AROUND? }
    then
      begin
        if MemAvail + test >= MinMem     { IF ENOUGH MEMORY LEFT }
          then
            begin
              new(p);
              with p^ do  { PROBABLY UNNECESSARY - VESTIGE FROM STRNGPAK }
                for I := 1 to chunksize do line[I] := ' ';
            end
          else
            fail := true  { NOT ENOUGH MEMORY }
      end
    else
      begin   { USING OLD CHUNK RATHER THAN MAKING A NEW ONE }
        p := availd;
        availd := availd^.next
      end
end;

procedure disposed (p : ptr);
begin
  p^.next := availd;    { PUT UNNEEDED CHUNK WHERE NEWD CAN RECYCLE IT }
  availd := p
end;

procedure rewrited (var d : document);
var
  fail : boolean;                  { PREPARE DOCUMENT TO ACCEPT INPUT }
begin                                 { FROM TEXTFILE OR KEYBOARD }
  with d do
    begin
      if start = nil
        then
          begin
            newd (start,fail);
            start^.next := nil
          end;
        current := start;
        position := 1;
        chunkno := 0;
        length := 0;
        status := writing
    end
end;

procedure resetd (var d: document);
var
  c : ptr;                      { PREPARE DOCUMENT TO BE READ FROM }
begin
  with d do
    begin
      if status = writing
        then
          begin
            length := length + position;
            c := current^.next;
            current^.next := nil;
            while c <> nil do
              begin
                current := c^.next;
                disposed (c);
                c := current
              end
          end;
      current := start;
      position := 1;
      chunkno := 0;
      status := reading;
      if current <> nil
        then
          w := current^.line[1]
        else
          w := ' '
    end
end;

function lengthd (d : document) : natural;   { HOW MANY LINES IN DOCUMENT ? }
begin
  resetd (d);
  lengthd := d.length
end;

function eofd (d : document) : boolean;      { IS NEXT LINE THE LAST ? }
begin
  with d do
    eofd := (length + 1) = chunkno * chunksize + position
end;

procedure putd (var d : document);
var
  fail : boolean;                   { JUST LIKE A FILE PUT }
begin                             { ACCEPT THE PRESENT INPUT }
  with d do                    { AND PREPARE TO ACCEPT THE NEXT }
    begin
      if status = reading
        then
          documenterror(1);
      if position = chunksize     { GO TO NEXT CHUNK }
        then
          begin
            if current^.next = nil   { IF NO NEXT CHUNK THEN }
              then
                begin
                  newd (current^.next,fail);  { ALLOCATE NEW CHUNK }
                  if fail
                    then
                      exit;
                  current^.next^.next := nil
                end;
            current := current^.next;       { SET RECORD TO REFLECT }
            chunkno := chunkno + 1;           { NEW CHUNK POSITION }
            length := length + chunksize;
            position := 1
          end
        else
          position := position + 1; { NO NEW CHUNK NEEDED -- MOVING DOWN ARRAY}
      current^.line[position] := w;   { LINE AT POSITION = DOCUMENT 'WINDOW' }
      w := ' ';      { RESET WINDOW }
    end
end;

procedure getd (var d : document);
begin
  with d do                          { JUST LIKE FILE GET }
    begin                             { SEE PUTD COMMENTS }
      if status = writing
        then
          documenterror(2);
      if eofd (d)
        then
          documenterror(3);
      if position = chunksize
        then
          begin
            current := current^.next;
            chunkno := chunkno + 1;
            position := 1
          end
        else
          position := position + 1;
      if current <> nil
        then
          w := current^.line[position]  { WINDOW = CURRENT POSITION }
        else
          w := ' ';
    end
end;

procedure opend (var d: document);
begin                                    { PREPARE ALREADY EXISTING }
  with d do                      { DOCUMENT TO BE EITHER RESET OR REWRITTEN }
    begin
      length := 0;
      chunkno := 0;
      position := 0;
      start := nil;
      current := nil;
      status := notready;
      w := ' '
    end
end;

procedure closed (var d : document);
begin
  with d do                     { DE - ALLOCATE ALL CHUNKS IN DOCUMENT }
    while start <> nil do       { IF NOT SAVED TO DISK, IT IS HISTORY! }
      begin
        current := start^.next;
        disposed (start);
        start := current
      end
end;

function MemLeft ( report : integer ) : real;
var
  test,
  bytesleft : real;
begin
  if MemAvail < 0          { MemAvail REPORTS PARAGRAPHS IN INTEGER FORM }
    then                  { IF MORE THAN 32767 PARAGRAPHS (16 BYTES) THEN }
      test := Invert        { IT WILL RETURN A NEGATIVE NUMBER -- OOPS }
    else                           { ADDING INVERT WILL FIX THIS }
      test := 0.0;
  bytesleft := (MemAvail + test) * 16;
  MemLeft := bytesleft;
  if report <> 0
    then
      begin
        textcolor (0);
        textbackground (15);
        gotoxy (52,25);
        write (bytesleft : 7 : 0,' bytes RAM available ');
        textcolor (15);
        textbackground (0);
        ClrEol
      end
end;

procedure readfile (var from : text; var d : document);
var
  I     : integer;                     { LOADS DOCUMENT FROM DISK FILE }
  dummy : char;                        { CHECKS FOR SUFFICIENT MEMORY }
  test  : real;
begin
  rewrited (d);
  while (MemLeft (1) >= MinMem) and not EOF (from) do
    begin
      readln (from,d.w);
      putd (d)
    end;
  ClrScr;
  if (MemAvail < MinMem) and (MemAvail > 0.0)
    then
      begin
        textcolor (20);
        gotoxy (15,13);
        write (' ***** MEMORY FULL --- DOCUMENT TRUNCATED *****');
        gotoxy (15,15);
        write (' ***** ONLY ABLE TO LOAD FIRST ',lengthd (d),' LINES *****');
        gotoxy (15,17);
        write (' PRESS ANY KEY TO CONTINUE . . . ');
        textcolor (15);
        read (dummy)
      end
end;

procedure readscr (var d : document);
begin
  readfile (input,d);
  reset (input)    { makes it possible to have more than one active document }
end;

procedure writefile (var onto : text; d : document);
begin
  resetd (d);              { WRITES DOCUMENT TO A TEXT FILE i.e LST }
  while not eofd (d) do
    begin
      writeln (onto,d.w);
      getd (d)
    end
end;

procedure writescr (d : document);
begin                                 { WRITES TO SCREEN }
  writefile (output,d)
end;

procedure fastget (var d : document; pos : natural);
var
  chunkpos, I : integer;                  { LOCATES A LINE IN A DOCUMENT BY }
begin                                  { LINE NUMBER - FASTER THEN SEQUENTIAL }
  chunkpos := pos div chunksize;                  { CALLS TO GETD }
  if d.chunkno >= chunkpos
    then
      begin
        resetd (d);
        while d.chunkno < chunkpos do
          begin
            d.current := d.current^.next;
            d.chunkno := d.chunkno + 1
          end
      end;
  d.position := pos mod chunksize;
  while (d.position + (d.chunkno * chunksize)) <= pos do
    getd (d);    { NEVER MORE THAN CHUNKSIZE CALLS TO GETD }
end;

procedure writepos (var d : document; pos : natural);
var
  temp : short;                { WRITES THE CORRECT CURRENT 80 CHARACTERS }
begin                             { FROM ANY DESIRED POSITION TO SCREEN }
  fastget (d,pos);
  temp := copy(d.w,left,80);
  write(temp,7)
end;

procedure suppress (var d : document);
const
  space = '';

var
  spaces : boolean;               { ELIMINATES TRAILING BLANK LINES WHEN }
  mark,I,L : natural;                  { CONCANTENATING DOCUMENTS }

begin
  L := lengthd (d);
  mark := 0;
  resetd (d);
  spaces := false;
  for I := 1 to L do
    begin
      if d.w = space
        then
          begin
            if not spaces
              then
                begin
                  spaces := true;
                  mark := I
                end
          end
        else
          begin
            spaces := false;
            mark := 0
          end;
      getd (d);
    end;
  if mark > 0
    then
      d.length := mark - 1;
  resetd (d)
end;

procedure assignd (var d1 : document; d2 : document);
begin
  rewrited (d1);
  resetd (d2);
  while not eofd (d2) do
    begin
      d1.w := d2.w;
      putd (d1);
      getd (d2)
    end
end;

function compare (d1 : document; r : relation; d2 : document) : boolean;
var
  less,equal : boolean;
  ld1,ld2 : natural;

begin
  ld1 := lengthd (d1);
  ld2 := lengthd (d2);
  resetd (d1);
  resetd (d2);
  equal := ld1 = ld2;
  less := false;
  while (equal and not less) and not eofd (d1) and not eofd (d2) do
    begin
      equal := d1.w = d2.w;
      less := d1.w < d2.w;
      getd (d1);
      getd (d2)
    end;
  case r of
       before : compare := less;
       beforeorequalto : compare := less or equal;
       equalto : compare := equal;
       afterorequalto : compare := not less or equal;
       after : compare := not less;
       notequalto : compare := not equal
  end
end;

procedure linetodocument (line : stype; var d : document);
begin
  rewrited (d);
  d.w := line;
  putd (d)
end;

procedure copyd (var d1 : document; d2 : document);
begin
  resetd (d2);
  while not eofd (d2) do
    begin
      d1.w := d2.w;
      putd (d1);
      getd (d2)
    end
end;

procedure append (var d1 : document; d2 : document);
var
  dt : document;

begin
  dcreate (dt);
  rewrited (dt);
  copyd (dt,d1);
  copyd (dt,d2);
  rewrited (d1);
  copyd (d1,dt)
end;

procedure extract ( src : document; from : cardinal;
                    lngth : natural; var object : document);
var
  I : cardinal;
  dst : document;

begin
  dcreate (dst);
  if (lengthd (src) < (from + lngth - 1))
    then
      documenterror (5);
  resetd (src);
  rewrited (dst);
  for I := 1 to (from - 1) do
    getd (src);
  for I := 1 to lngth do
    begin
      dst.w := src.w;
      putd (dst);
      getd (src)
    end;
  copyd (object,dst)
end;

procedure insert (var dst : document; src : document; after : natural);
var
  I : cardinal;
  dt : document;

begin
  dcreate (dt);
  if (lengthd (dst) < after)
    then
      documenterror (6);
  resetd (dst);
  rewrited (dt);
  for I := 1 to after do
    begin
      dt.w := dst.w;
      putd (dt);
      getd (dst)
    end;
  copyd (dt,src);
  while not eofd (dst) do
    begin
      dt.w := dst.w;
      putd (dt);
      getd (dst)
    end;
  rewrited (dst);
  copyd (dst,dt)
end;

procedure delete (var dst : document; from : cardinal; lngth : natural);
var
  I : cardinal;
  dt : document;

begin
  dcreate (dt);
  if (lengthd (dst) < (from + lngth - 1))
    then
      documenterror (4);
  resetd (dst);
  rewrited (dt);
  for I := 1 to (from - 1) do
    begin
      dt.w := dst.w;
      putd (dt);
      getd (dst)
    end;
  for I := 1 to lngth do
    getd (dst);
  while not eofd (dst) do
    begin
      dt.w := dst.w;
      putd (dt);
      getd (dst)
    end;
  rewrited (dst);
  copyd (dst,dt)
end;

function find (d1,d2 : document) : natural;
var
  m,n : natural;
  I : cardinal;
  object : document;
  state : (scanning,found,notfound);

begin
  dcreate (object);
  m := lengthd (d1);
  n := lengthd (d2);
  if (n = 0) or (m < n)
    then
      begin
        find := 0
      end
    else
      begin
        I := 1;
        state := scanning;
        while (state = scanning) do
          begin
            extract (d1,I,n,object);
            if (compare (object,equalto,d2))
              then
                begin
                  state := found;
                  find := I
                end
              else
                begin
                  I := I + 1;
                  if ((m - I + 1) < n)
                    then
                      begin
                        state := notfound;
                        find := 0
                      end;
                end
          end
      end
end;

procedure concat (var d1 : document; d2 : document);
begin
  suppress (d1);
  append (d1,d2)
end;

procedure addline (var d : document; line : stype);
var
  dt : document;

begin
  dcreate (dt);
  linetodocument (line,dt);
  concat (d,dt)
end;

procedure dcreate;
var
  temp : document;

begin
  with temp do
    begin
      w := ' ';
      length := 0;
      position := 0;
      start := nil;
      current := nil;
      chunkno := 0;
      status := notready
    end;
  d := temp
end;

procedure init_document_pack;
begin
  availd := nil;
  left := 1
end;
