Unit STIHASH;                               {Generic Hash Table             }
{$B-}
{$V-}

Interface

Const
  MAXENTRY  = 23;                           {maximum entries : prime wins   }

Type
  EntryPtr  = ^EntryRec;                    {pointer to an entry            }
  EntryRec  = record                        {this is an entry               }
		Data : pointer;             {this is the data               }
                Key  : pointer;             {this is the key                }
                Next : EntryPtr;            {this is the next entry         }
              end;
  Entries   = array[0..MAXENTRY] of EntryPtr; {entry array                  }
  UsedFlags = array[0..MAXENTRY] of boolean;{are the entries used           }
  HashTable = record                        {this is the hash table type    }
                Entry : Entries;            {these are the entries          }
                Used  : UsedFlags;          {these are the used flags       }
              end;
  HashTPtr  = ^HashTable;                   {pointer to a hash table        }

Var
  HTable    : HashTPtr;                     {this is the current hashtable  }
                                            {allows same unit to handle many}
                                            {tables                         }


Procedure STI_EntryInit(    Var Entry : EntryRec; DataSize,KeySize : word);
Procedure STI_EntryCreate(  Var Entry : EntryRec);
Procedure STI_EntryLink(    Var Entry : EntryRec; NewRec : EntryRec);
Procedure STI_EntryDestroy( Var Entry : EntryRec; KeySize,DataSize : word);
Procedure STI_EntrySet_Key(Var Entry  : EntryRec; Var Key;  Size : word);
Procedure STI_EntrySet_Data(Var Entry : EntryRec; Var Data; Size : word);
Procedure STI_EntryGet_Data(Var Entry : EntryRec; Var Data; Size : word);

Procedure STI_HashTableCreate;
Procedure STI_HashTableEnter(E            : EntryRec;
                            KeySize       : word;
                            Var Duplicate : Boolean);
Procedure STI_HashTableRetrieve(TheKey    : EntryRec;
                                KeySize   : word;
                                Var Found : Boolean;
                                Var E     : EntryRec);
Procedure STI_HashTableDestroy(KeySize,DataSize : word);


Implementation

{---------------------------------------------------------------------------}
{           Procedures for handling the entries                             }
{---------------------------------------------------------------------------}

Procedure STI_EntryInit(Var Entry : EntryRec; DataSize,KeySize : word);

begin
  GetMem(Entry.Data,DataSize);              {get the memory                 }
  FillChar(Entry.Data^,DataSize,#32);       {null the memory                }
  GetMem(Entry.Key,KeySize);                {get the memory                 }
  FillChar(Entry.Key^,KeySize,#32);         {null the memory                }
end;

{---------------------------------------------------------------------------}

Procedure STI_EntrySet_Data(Var Entry : EntryRec; Var Data; Size : word);

begin
  Move(Data,Entry.Data^,Size);              {reassign the data pointer      }
end;

{---------------------------------------------------------------------------}

Procedure STI_EntrySet_Key(Var Entry : EntryRec; Var Key; Size : word);

begin
  Move(Key,Entry.Key^,Size);                {reassign the key pointer       }
end;

{---------------------------------------------------------------------------}

Procedure STI_EntryGet_Data(Var Entry : EntryRec; Var Data; Size : word);

begin
  move(Entry.Data^,Data,Size);              {reassign the data pointer      }
end;

{---------------------------------------------------------------------------}

Procedure STI_EntryCreate(Var Entry : EntryRec);

Begin
  Entry.Next := NIL;                        {null the next entry            }
  Entry.Key  := NIL;                        {null the key                   }
  Entry.Data := NIL;                        {nulL the data                  }
End;

{---------------------------------------------------------------------------}

Procedure STI_EntryLink(Var Entry : EntryRec; NewRec : EntryRec);

Begin
  New(Entry.Next);                          {get the next entry             }
  Entry.Next^ := NewRec                     {assign the new entry to it     }
End;

{---------------------------------------------------------------------------}

Procedure STI_EntryDestroy(Var Entry : EntryRec; KeySize,DataSize : word);

Begin
  if Entry.Key <> NIL then                  {check to see if key is used    }
    freemem(pointer(Entry.Key),KeySize);    {if so, release the memory      }
  Entry.Key := NIL;                         {kill the key                   }
  if Entry.Data <> NIL then                 {check to see if data is used   }
    freemem(pointer(Entry.Data),DataSize);  {if so, get rid of it           }
  Entry.Data := NIL;                        {kill the data                  }
End;

{---------------------------------------------------------------------------}
{         Procedures for handling the Hash Table                            }
{---------------------------------------------------------------------------}

Procedure STI_HashTableCreate;

Var
  Loop : Word;                              {general loop variable          }

Begin
  For Loop := 0 to MaxEntry-1 do            {loop to maxentries             }
    Begin
      New(HTable^.Entry[Loop]);             {get a new entry                }
      STI_EntryCreate(HTable^.Entry[Loop]^); {initialise the entry         }
      HTable^.Used[Loop] := False;          {mark the entry unused          }
    End
End;

{---------------------------------------------------------------------------}

Function Hash(Var TheKey; S : word) : Word; {the actual hash function       }
                                            {returns a number to MAXENTRY   }
Var
  Temp : word;                              {temporary varaibel             }
  Loop : Word;                              {general loop varaible          }
  Key  : array[1..65535] of byte absolute TheKey;

Begin
  Temp := 0;                                {null temp                      }
  For Loop := 1 to S do                     {loop on string length          }
    Temp := Temp + Key[Loop];               {build up temp                  }
  Hash := word(temp MOD (MaxEntry));        {simple hash function but good  }
End;

{---------------------------------------------------------------------------}

Function Compare(Var S1,S2; S : word) : Boolean;

Var
  Loop   : byte;
  C1     : array[1..65535] of char absolute S1;
  C2     : array[1..65535] of char absolute S2;

Begin
  Compare := TRUE;                          {default to true               }
  for Loop := 1 to S do                     {loop over arrays              }
    begin
      if C1[loop] <> C2[loop] then          {are they the same             }
        begin
          Compare := FALSE;                 {no,set compare to false & exit}
          Exit;
        end;
    end;
End;

{---------------------------------------------------------------------------}

Procedure STI_HashTableEnter(E : EntryRec; KeySize : word; Var Duplicate : Boolean);

Var
  Temp  : EntryPtr;                         {temporary entry                }
  Ndex  : Word;                             {index to hash table            }
  Done  : Boolean;                          {are we finished                }
  Comp  : Boolean;                          {key comparison variable        }

Begin
  Comp := False;
  Duplicate := False;                       {default to no duplicate        }
  Ndex := Hash(E.Key^,KeySize);             {get the index to the table     }
  If Not HTable^.Used[Ndex] Then            {is this entry used             }
    Begin
      HTable^.Entry[Ndex]^ := E;            {no so poke E in there          }
      HTable^.Used[Ndex] := True            {and set used to true           }
    End
  Else                                      {yes it is used                 }
    Begin
      Done := False;                        {we are not done yet            }
      Temp := HTable^.Entry[Ndex];          {we give the entry to temp      }
      While Not Done do
        Begin
          Comp := Compare(Temp^.Key^,E.Key^,KeySize);
          If Comp Then                      {keys are the same              }
            Begin
              Duplicate := True;            {so this is a duplicate         }
              Done := True;                 {and we are finished            }
            End
          Else                              {the keys are different         }
            Begin
              While Temp^.Next <> Nil do    {look for the same key          }
                Begin
                  Temp := Temp^.Next;       {check the next entry           }
                  Comp :=
                    Compare(Temp^.Key^,E.Key^,KeySize);
                  If Comp Then
                    Duplicate := True       {same keys                      }
                End;
              Done := True                  {all done                       }
            End
        End;
      If Not Duplicate Then                 {is it a duplicate              }
             STI_EntryLink(Temp^,E);        {no so enter it in the table    }
    End
End;

{---------------------------------------------------------------------------}

Procedure STI_HashTableRetrieve(TheKey  : EntryRec; KeySize : word; Var Found : Boolean;
                           Var E : EntryRec);
Var
  Temp : EntryPtr;                          {this is the entry              }
  Ndex : Word;                              {this is the index              }
  Done : Boolean;                           {are we finished                }
  Comp : Boolean;                           {comparison variable            }

Begin
  Comp  := False;
  Found := False;                           {default to not found           }
  Ndex := Hash(TheKey.Key^,KeySize);        {get the index                  }
  If Not HTable^.Used[Ndex] Then            {is this entry used             }
     Exit                                   {no so get out of here          }
  Else                                      {yes it is used                 }
    Begin
      Done := False;                        {we are not finished yet        }
      Temp := HTable^.Entry[Ndex];          {assign the entry to temp       }
      Comp := Compare(Temp^.Key^,TheKey.Key^,KeySize);
      While (Not Done) and (Not Comp) do    {loop until done or found       }
        begin
          If Temp^.Next = Nil Then          {are we at the end of the line  }
            Done := True                    {yes so get out                 }
          Else
            Temp := Temp^.Next;             {no so move on down the line    }
          Comp := Compare(Temp^.Key^,TheKey.Key^,KeySize)
        end;
      Comp := Compare(Temp^.Key^,TheKey.Key^,KeySize);
      If Not Comp Then
        Exit                                {Key not found                  }
      Else
        Begin
          E := Temp^;                       {the entry found is temp        }
          Found := True                     {oh yes we found it             }
        End
    End
End;

{---------------------------------------------------------------------------}

Procedure STI_HashTableDestroy(KeySize,DataSize : word);
Var
  Temp1 : EntryPtr;                         {temporary entry                }
  Temp2 : EntryPtr;                         {temorary entry                 }
  Ndex  : Word;                             {the index to the hash table    }

Begin
  For Ndex := 0 to MaxEntry-1 do            {loop for number of entries     }
    Begin
      While HTable^.Entry[Ndex] <> Nil do   {is the entry allocated         }
        Begin
          Temp1 := HTable^.Entry[Ndex];     {yes, give it to temp1          }
          Temp2 := EntryPtr(Temp1)^.Next;   {give temp2 the next one        }
          HTable^.Entry[Ndex] := Temp2;     {the ntery now points to next   }
          if Temp1 <> Nil then              {is temp1 used                  }
            begin
              STI_EntryDestroy(Temp1^,KeySize,DataSize);
              dispose(Temp1);
            end;
                                            {yes so free the memory         }
        End
    End
End;

{---------------------------------------------------------------------------}

begin
end.