{TOADLONG.INC}
(* Copyright (C) 1988 David P Kirschbaum  All Rights Reserved *)

TYPE
  long_int = RECORD CASE INTEGER OF  {lots of ways to look at a long}
              0 : (byte0,byte1,byte2,byte3 : BYTE);
              {     al    ah    dl    dh }
              1 : (lo,hi : INTEGER);
             END;

  numtype = (realtyp,inttyp,longtyp);

  {I'm TYPEing lots of different string types here.
   Actually, you can go with fewer and use the compiler instruction
   V- to relax string type checking.
  }
  Str4  = STRING[4];
  Str5  = STRING[5];
  Str8  = STRING[8];
  Str9  = STRING[9];
  Str10 = STRING[10];
  Str11 = STRING[11];
  Str12 = STRING[12];
  Str14 = STRING[14];
  Str64 = STRING[64];

(* This is how you'd define a constant long integer.
   Kinda handy to have some typed constants around for
   zeroing long_int variables, etc.
*)

CONST
  LONG_ZERO : long_int = (lo:0     ; hi:0);
  LONG_ONE  : long_int = (lo:1     ; hi:0);
  MAXLONGINT : long_int = (lo:$FFFF; hi:$FFFF);
  MAXREALINT = 4294967295.0;  {the largest real that can fit in a long integer}

VAR
  flags : INTEGER;                {global "flags" register}
  tlong : long_int;               {example of global long int variable}

(* Some flag functions *)

PROCEDURE Clc;
  {Clear bit 0 of our flags integer, just like assembler CLC.}
  BEGIN
    Inline(
  $80/$26/>FLAGS/$FE     {and  byte [<flags],$FE  ;clear bit 0}
);
  END;  {of Clc}

PROCEDURE Stc;
  {Set bit 0 of our flags integer, just like assembler STC.}
  BEGIN
    Inline(
   $80/$0E/>FLAGS/$01    {or   byte [<flags],1    ;set bit 0}
);
  END;  {of Stc}

FUNCTION CF : BOOLEAN;
  {Return TRUE if carry bit of global flags integer is set,
   else return FALSE.
   In assembler:
     JC wherever  ; jump if carry flag set
   In Turbo:
     IF Carry THEN Goto
  }
  BEGIN
    CF := (flags AND 1 = 1);
  END;  {of CF}

FUNCTION ZF : BOOLEAN;
  {Return TRUE if zero bit of global flags integer is 0,
   else return TRUE.
   In assembler:
    JNZ Beyond  ;jump if zero flag not set
      do this
    Beyond:
   In Turbo:
     IF ZF THEN BEGIN
       do this
     END;
  }
  BEGIN
    ZF := (flags AND 64 = 64);  {bit 6, ZF, is set, e.g., ZERO}
  END;  {of ZF}


(* Toad Hall unsigned long integer procedures and functions *)

(* No
  AAA
  AAD
  AAM
  AAS
  ADC  We COULD add this as a long function, now that our carry is
       extended to long addition, subtraction, etc.  Maybe later.
*)

{ ADD }

PROCEDURE Long_Add(VAR long, addn : long_int);
  { Adds two long integers together, producing new long.
   Think of it as L1 := L1 + L2.
  }
  BEGIN
    Inline(
{;ADD1  adding global flags}
  $8C/$DB          { mov  bx,DS        ;preserve DS}
  /$C5/$B6/>ADDN   { lds  si,>addn[bp] ;nr we'll add in}
  /$FC             { cld               ;go forward}
  /$AD             { lodsw             ;snarf addn.lo}
  /$8B/$0C         { mov  cx,[si]      ;snarf addn.hi}
  /$09/$C0         { or   ax,ax        ;0?}
  /$75/$02         { jne  Add1         ; nope, continue}
  /$E3/$0B         {  jcxz NoAdd       ; 0, forget it}
                   {Add1:}
  /$C4/$BE/>LONG   { les  di,>long[bp] ;gonna stuff it back}
  /$26             { ES:}
  /$01/$05         { add  [di],ax      ;add to long.lo}
  /$26             { ES:}
  /$11/$4D/$02     { adc  [di+2],cx    ;add w/carry to long.hi}
                   {NoAdd:}
  /$8E/$DB         { mov  DS,bx        ;restore DS}
  /$9F             { lahf              ;get the flags}
  /$88/$26/>FLAGS  { mov  [<flags],ah  ; and save in global}
);
  END;  { of Long_Add}


{ ADD }

PROCEDURE Long_Add_I(VAR long : long_int; i : INTEGER);
  { Add an unsigned integer to an unsigned long integer,
    return the long integer.
    Think of it as L1 := L1 + i
  }
  BEGIN
    Inline(
{;ADD_I1  adding global flags}
  $C4/$BE/>LONG    { les  di,>long[bp]  ;long we'll add to}
  /$8B/$86/>I      { mov  ax,>i[bp]     ;integer to add}
  /$26             { ES:}
  /$01/$05         { add  [di],ax       ;add to long.lo}
  /$73/$06         { jnc  Exit          ;no carry, ok}
  /$26             { ES:}
  /$81/$45/$02
  /$01/$00         { add  word [di+2],1 ;bump long.hi, fix CF}
                   {Exit:}
  /$9F             { lahf               ;get flags reg}
  /$88/$26/>FLAGS  { mov  [<flags],ah   ;update global flags}
);
  END;  {of Long_Add_I}


{ AND }

PROCEDURE Long_And(VAR long, ander : long_int);
  {AND first unsigned long integer with a second long int,
   return the first long int.
   Think of it as L1 := L1 AND ander
  }
  BEGIN
    Inline(
  $8C/$DB          { mov  bx,DS          ;preserve DS}
  /$C5/$B6/>ANDER  { lds  si,>ander[bp]  ;the AND long int vector}
  /$C4/$BE/>LONG   { les  di,>long[bp]   ;target long int vector}
  /$FC             { cld                 ;insure fwd}
  /$AD             { lodsw               ;ander.lo}
  /$26             { ES:}
  /$21/$05         { and  [di],ax        ;AND with long.lo}
  /$AD             { lodsw               ;ander.hi}
  /$26             { ES:}
  /$21/$45/$02     { and  [di+2],ax      ;AND with long.hi}
  /$8E/$DB         { mov  DS,bx          ;restore}
);
  END;  {of Long_And}


(* No
  CBW  We could do this (creating a long integer from an byte,
       extending the byte's sign through the long's lo and hi words.
       But why?
*)

{ CMP }

FUNCTION Long_Cmp(VAR long1 : long_int; Op : Rel_Op;
                  VAR long2 : long_int) : BOOLEAN;
  { Toad Hall comments:
    With Rel_Op = (Eq, Gt, Lt, Ne, Ge, Le.)
                    0   1   2   3   4   5
    It's silly to call this for an Eq or Ne test, really,
    since even signed integers can be easily tested for equality
    (e.g., if x = y ) without needing this slow call (with all these
    parameters) for unsigned integer tests.
    Just do something inline, like you'll see below in the Eq and Ne
    tests.  Faster, cleaner, tighter.
  }
  VAR Boo : BOOLEAN;
  BEGIN
    Inline(
  $1E              {  push DS           ;save DS}
  /$31/$C0         {  xor  ax,ax        ;clear msb}
  /$8A/$46/<OP     {  mov  al,<Op[bp]   ;get operator}
  /$89/$C7         {  mov  di,ax        ;save in di as word}
  /$FC             {  cld               ;insure fwd}
  /$C5/$B6/>LONG1  {  lds  si,>long1[bp] ;get long1 address}
  /$AD             {  lodsw             ;long1.lo}
  /$89/$C3         {  mov bx,ax         ;bx=long1.lo}
  /$8B/$0C         {  mov cx,[si]       ;cx=long1.hi}
  /$C5/$B6/>LONG2  {  lds si,>long2[bp] ;get long2 address}
  /$AD             {  lodsw       ;long2.lo}
  /$8B/$14         {  mov dx,[si] ;long2.hi}
  /$39/$C3         {  cmp bx,ax   ;set flags for lo test}
  /$9C             {  pushf       ;save on stack}
  /$39/$D1         {  cmp cx,dx   ;set flags for hi test}
  /$9C             {  pushf       ;save on stack}
  /$09/$FF         {  or  di,di   ;0?}
  /$74/$49         {  jz  Eq}
  /$4F             {  dec di      ;1?}
  /$74/$39         {  jz  Gt}
  /$4F             {  dec di      ;2?}
  /$74/$2B         {  jz  Lt}
  /$4F             {  dec di      ;3?}
  /$74/$1D         {  jz  Ne}
  /$4F             {  dec di      ;4?}
  /$74/$0D         {  jz  Ge}
                   {;gotta be 5, Le}
  /$9D             {  popf              ;hi test flags}
  /$74/$05         {  je   LeLo         ;hi =}
  /$58             {   pop  ax          ;clear lo flags}
  /$77/$40         {   ja   FALSE       ; >}
  /$EB/$42         {   jmp  short True  ; <}
                   {LeLo:}
  /$9D             {  popf              ;lo test flags}
  /$77/$3B         {  ja   False        ; >}
  /$EB/$3D         {  jmp  short True   ; <=}
                   {Ge:}
  /$9D             {  popf              ;hi test flags}
  /$74/$05         {  je  GeLo          ; =}
  /$58             {   pop ax           ;clear lo flags}
  /$72/$33         {   jb  False        ; <}
  /$EB/$35         {   jmp short True   ; >}
                   {GeLo:}
  /$9D             {   popf             ;lo test flags}
  /$72/$2E         {   jb  False        ; <}
  /$EB/$30         {   jmp short True   ; >=}
                   {Ne:}
  /$9D             {  popf              ;hi test flags}
  /$74/$03         {  je   NeLo         ; =}
  /$58             {   pop  ax          ;clear lo flags}
  /$EB/$2A         {   jmp  short True  ; <>}
                   {NeLo:}
  /$9D             {  popf              ;lo test flags}
  /$74/$23         {  je   False        ; =}
  /$EB/$25         {  jmp  short True   ; <>}
                   {Lt:}
  /$9D             {  popf              ;hi test flags}
  /$76/$03         {  jbe  LtLo         ; <=}
  /$58             {   pop ax           ;clear lo flags}
  /$EB/$1B         {   jmp short False  ; >}
                   {LtLo:}
  /$9D             {  popf              ;lo test flags}
  /$73/$18         {  jnb  False        ; >=}
  /$EB/$1A         {  jmp  short True   ; <}
                   {Gt:}
  /$9D             {  popf              ;hi test flags}
  /$74/$05         {  je  GtLo          ; =}
  /$58             {   pop ax           ;clear lo flags}
  /$72/$10         {   jb  False        ; <}
  /$EB/$12         {   jmp short True   ; >}
                   {GtLo:}
  /$9D             {  popf              ;lo test flags}
  /$76/$0B         {  jna  False        ; <=}
  /$EB/$0D         {  jmp  short True   ; >}
                   {Eq:}
  /$9D             {  popf              ;hi test flags}
  /$74/$03         {  je  EqLo          ; =}
  /$58             {   pop ax           ;clear lo flags}
  /$EB/$03         {   jmp short FALSE  ;<>}
                   {EqLo:}
  /$9D             {  popf              ;lo test flags}
  /$74/$04         {  je True           ; =}
                   {False:}
  /$30/$C0         {  xor al,al         ;return false}
  /$EB/$02         {  jmp short Exit}
                   {True:}
  /$B0/$01         {  mov  al,1         ;return true}
                   {Exit:}
  /$1F             {  pop  DS           ;restore DS}
  /$88/$46/<BOO    {  mov  <boo[bp],al  ;stuff boolean result}
);
    Long_Cmp := Boo;  {Pick up local for return}
  END;  {of Long_Cmp}

(* No
  CWD .. though that's an appropriate function for long ints.
         If only I could figure out WHY ...
  DAA    Ugh...
  DAS
*)

{ DEC }

PROCEDURE Long_Dec(VAR long : long_int);
  {DECrement a long int.
   Think of it as L1 := PRED(L1)
  }
  BEGIN
(* In Turbo:
    Long_Sub_I(long, 1);
*)
    Inline(
{;DEC1  adding global flags (ZF only)}
  $C4/$BE/>LONG    { les  di,>long[bp] ;long we'll decr}
  /$26             { ES:}
  /$81/$2D/$01/$00 { sub  word [di],1  ;decr long.lo}
  /$73/$04         { jnb  Exit}
  /$26             { ES:}
  /$FF/$4D/$02     { dec  word [di+2]  ;if borrow (CF set)}
                   {Exit:}
  /$9F             { lahf              ;get flags}
  /$88/$26/>FLAGS  { mov  [<flags],ah  ;update global flags}
);
  END;  {of Long_Dec}


{DIV or /}

PROCEDURE Long_Div_I(VAR dvdnd : long_int; i : INTEGER);
  {Divide a long integer by an integer, return the
   long integer.  If any overflow or DIV 0, returns with
   Carry flag set, long unchanged.
   Otherwise truncated quotient is returned in long.
   Think of it as L1 := L1 / i
  }
  BEGIN
Inline(
{;DIV_I3  adding global flags, remainder still wrong}
  $1E              {  push DS        ;save DS}
  /$FC             {  cld            ;insure fwd}
  /$31/$C9         {  xor  cx,cx     ;cx=0, clear ZF,CF}
  /$9C             {  pushf          ;save flags on stack}
  /$8B/$86/>I      {  mov  ax,>i[BP] ;get the divisor}
  /$3D/$01/$00     {  cmp  ax,1      ;test divisor}
  /$74/$46         {  je   NoDiv     ;n/1=n, no action}
  /$72/$38         {  jb   StufQuot  ;n/0=illegal, no change, CF}
  /$89/$C3         {  mov  bx,ax     ;bx=divisor}
  /$C5/$B6/>DVDND  {  lds  si,>dvdnd[bp]   ;get dividend's address}
  /$AD             {  lodsw          ;dvdnd.lo, bump si}
  /$8B/$14         {  mov  dx,[si]   ;dvdnd.hi}
  /$31/$FF         {  xor  di,di     ;get a handy 0}
  /$39/$DA         {  cmp  dx,bx     ;hi >= divisor?}
  /$73/$0E         {  jae  Div32     ; yep, handle special}
  /$09/$D2         {   or  dx,dx     ; dvdnd.hi =0?}
  /$75/$06         {   jne Div16     ; nope, do 16-bit div}
  /$39/$D8         {    cmp ax,bx    ;  dvdnd=0 or < divisor?}
  /$72/$21         {    jb  StufQuot ;  n/(>n)=illegal, no change, CF}
  /$74/$13         {    je  Make1    ;  n/n=1, ZF}
                   {Div16:}
  /$F7/$F3         {  div  bx        ;divide, maybe dx=remainder}
  /$EB/$18         {  jmp short DivDone ;done (clear dx)}
                   {Div32:}
  /$89/$C1         {   mov cx,ax    ;save dvdnd.lo}
  /$89/$D0         {   mov ax,dx    ;dvdnd.hi}
  /$31/$D2         {   xor dx,dx    ;cwd, kinda}
  /$F7/$F3         {   div bx       ;divide dvdnd.hi}
  /$91             {   xchg ax,cx   ;cx=quotient.hi,ax=dvdnd.lo}
  /$75/$0B         {   jnz  DivLo   ;remainder, go divide}
  /$39/$D8         {    cmp ax,bx   ; dvdnd.lo >= divisor?}
  /$77/$07         {    ja  DivLo   ; more, go divide it}
                   {Make1:}
  /$89/$F8         {    mov ax,di   ;assume less (di=0)}
  /$75/$05         {    jne DivDone ;less, dvdnd.lo = 0}
  /$40             {     inc ax     ; n/n=1}
  /$EB/$02         {     jmp short DivDone ; done}
                   {DivLo:}
  /$F7/$F3         {  div bx               ;divide, ignore remainder}
                   {DivDone:}
  /$89/$CA         {  mov dx,cx            ;cx=quotient.hi or 0}
  /$F8             {  clc                  ;insure CF clear}
                   {;ax=quotient.lo, dx=quotient.hi}
                   {StufQuot:}
  /$5B             {  pop  bx              ;clear flags from stack}
  /$9C             {  pushf                ;save new flags}
  /$72/$08         {  jb NoDiv             ;overflow, no change, Carry set}
  /$C4/$BE/>DVDND  {   les  di,>dvdnd[bp]  ; stuff quotient on long}
  /$AB             {   stosw               ; quotient.lo = ax}
  /$26             {   ES:}
  /$89/$15         {   mov  [di],dx        ; quotient.hi = dx}
                   {NoDiv:}
  /$58             {  pop  ax              ;get flags from stack}
  /$1F             {  pop  DS              ;restore DS}
  /$A2/>FLAGS      {  mov  [<flags],al     ;update global flags}
);
  END;  {of Long_Div_I}


PROCEDURE Long_Div(VAR dvdnd, dvsr : long_int);
  {Divide a long integer dividend by a long integer divisor.
   Trying to deal with overflow:
     If overflow, dividend is unchanged, global flags Carry bit set.
      1: If divisor.hi <> 0
      2: If divisor.lo > dividend
   Otherwise long quotient (truncated) is returned as dvdnd.
   Think of it as L1 := L1 / L2
  }
  BEGIN
    Inline(
{;DIV  adding global flags, fixing bug}
  $1E              {  push DS              ;save DS}
  /$FC             {  cld                  ;insure fwd}
  /$31/$C9         {  xor  cx,cx           ;cx=0, clear ZF,CF}
  /$9C             {  pushf                ;save cleared flags on stack}
  /$C5/$B6/>DVSR   {  lds  si,>dvsr[BP]    ;get long divisor's address}
  /$AD             {  lodsw                ;get dvsr.lo, bump si}
  /$89/$C3         {  mov  bx,ax           ;bx=dvsr.lo}
  /$8B/$0C         {  mov  cx,[si]         ;cx=dvsr.hi}
  /$C5/$B6/>DVDND  {  lds  si,>dvdnd[bp]   ;get dividend's address}
  /$AD             {  lodsw                ;ax=dvdnd.lo, bump si}
  /$8B/$14         {  mov  dx,[si]         ;dx=dvdnd.hi}
  /$39/$D1         {  cmp  cx,dx           ;dvsr > dvdnd?}
  /$77/$60         {  ja   JmpJmp          ; yep, no change, CF set}
  /$75/$0B         {  jne  NotSame         ;dvsr <> dvdnd}
  /$39/$D8         {   cmp ax,bx           ; lo's same?}
  /$75/$07         {   jne NotSame         ; nope}
  /$B8/$01/$00     {    mov ax,1           ;  n/n = 1}
  /$31/$D2         {    xor dx,dx          ; clear quotient.hi, CF}
  /$EB/$53         {    jmp short JmpJmp   ;done}
                   {NotSame:}
  /$09/$D2         {  or   dx,dx           ;dvdnd.hi=0?}
  /$75/$07         {  jne  NotZero         ;nope}
  /$09/$C0         {   or  ax,ax           ; dvdnd.lo=0?}
  /$75/$03         {   jne NotZero         ; nope}
  /$F9             {    stc                ;  bad, set CF}
  /$EB/$7B         {    jmp short StufQuot ;  no change, CF set}
                   {NotZero:}
  /$09/$C9         {  or   cx,cx           ;dvsr.hi=0?}
  /$75/$07         {  jne  NotZero2        ;nope}
  /$09/$DB         {   or  bx,bx           ; dvsr.lo=0?}
  /$75/$03         {   jne NotZero2        ; nope}
  /$F9             {    stc                ;  bad, set CF}
  /$EB/$70         {    jmp short StufQuot ;  no change, CF set}
                   {NotZero2:}
                   {;find common denominator w/shr's}
  /$BF/$01/$00     {  mov  di,1            ;constant for testing}
                   {DenomL:}
  /$85/$FB         {  test bx,di           ;dvsr odd?}
  /$75/$14         {  jnz  NoCommon        ;yep}
  /$85/$F8         {   test ax,di          ; dvdnd odd?}
  /$75/$10         {   jnz  NoCommon       ; yep}
                   {;ShR until somebody goes odd or dvsr zeroes}
  /$D1/$E9         {    shr cx,1           ; dvsr.hi}
  /$D1/$DB         {    rcr bx,1           ; dvsr.lo, CF => bit 7}
  /$75/$04         {    jne Denom1         ; dvsr.lo <> 0, continue}
  /$09/$C9         {     or cx,cx          ; dvsr.hi 0?}
  /$74/$5B         {     je StufQuot       ; done}
                   {Denom1:}
  /$D1/$EA         {    shr dx,1           ; dvdnd.hi}
  /$D1/$D8         {    rcr ax,1           ; dvdnd.hi, CF => bit 7}
  /$EB/$E8         {    jmp short DenomL   ; loop}
                   {NoCommon:}
  /$09/$C9         {  or  cx,cx            ;dvsr double-word?}
  /$74/$20         {  jz  DoDiv            ; nope, safe to divide}
                   {;double-word division via adds}
  /$55             {  push bp              ;ran out of regs}
  /$31/$ED         {  xor  bp,bp           ;quotient builds in bp}
  /$31/$F6         {  xor  si,si           ;clear accumulator.lo}
  /$31/$FF         {  xor  di,di           ; & acc.hi}
                   {AddL:}
  /$01/$DE         {  add  si,bx           ;acc.lo}
  /$11/$CF         {  adc  di,cx           ;acc.hi + CF}
  /$45             {  inc  bp              ;bump quotient, CF unchanged}
  /$72/$0A         {  jc   AddDone         ; hi carried, done}
  /$39/$D7         {   cmp di,dx           ;acc > dvdnd?}
  /$72/$F5         {   jb  AddL            ;not yet}
  /$77/$04         {   ja  AddDone         ;done}
  /$39/$C6         {    cmp si,ax          ; acc.lo > dvdnd.lo?}
  /$72/$EF         {    jb  AddL           ; not yet}
                   {AddDone:  ;integer quotient now in bp}
  /$89/$E8         {  mov  ax,bp           ;quotient.lo}
  /$5D             {  pop  bp              ;restore bp}
  /$31/$D2         {  xor  dx,dx           ;clear quotient.hi}
  /$48             {  dec  ax              ;adjust for excess inc, set ZF}
                   {JmpJmp:}
  /$EB/$31         {  jmp  short StufQuot  ;done}
                   {DoDiv:  ;normal doubleword/word div}
  /$31/$FF         {  xor  di,di           ;get a handy 0}
  /$39/$DA         {  cmp  dx,bx           ;hi >= divisor?}
  /$73/$10         {  jae  Div32           ; yep, handle special}
  /$09/$D2         {   or  dx,dx           ;dvdnd.hi =0?}
  /$75/$08         {   jne Div16           ; nope, do 16-bit div}
  /$39/$D8         {    cmp ax,bx          ;dvdnd=0 or < divisor?}
  /$77/$04         {    ja  Div16          ;save to divide}
  /$72/$21         {    jb  StufQuot       ;n/(>n)=illegal, no change, CF}
  /$EB/$13         {    jmp short Make1    ;n/n=1, ZF}
                   {Div16:}
  /$F7/$F3         {  div  bx              ;divide, maybe dx=remainder}
  /$EB/$18         {  jmp short DivDone    ;done (clear dx)}
                   {Div32:}
  /$89/$C1         {   mov cx,ax           ;save dvdnd.lo}
  /$89/$D0         {   mov ax,dx           ;dvdnd.hi}
  /$31/$D2         {   xor dx,dx           ;cwd, kinda}
  /$F7/$F3         {   div bx              ;divide dvdnd.hi}
  /$91             {   xchg ax,cx          ;cx=quotient.hi,ax=dvdnd.lo}
  /$75/$0B         {   jnz  DivLo          ;remainder, go divide}
  /$39/$D8         {    cmp ax,bx          ;dvdnd.lo >= divisor?}
  /$77/$07         {    ja  DivLo          ; more, go divide it}
                   {Make1:}
  /$89/$F8         {    mov ax,di          ;assume less (di=0)}
  /$75/$05         {    jne DivDone        ;less, dvdnd.lo = 0}
  /$40             {     inc ax            ;n/n=1}
  /$EB/$02         {     jmp short DivDone ;done}
                   {DivLo:}
  /$F7/$F3         {  div bx               ;divide, ignore remainder}
                   {DivDone:}
  /$89/$CA         {    mov dx,cx          ;cx=quotient.hi or 0}
  /$F8             {    clc                ;insure CF clear}
                   {;ax=quotient.lo, dx=quotient.hi}
                   {StufQuot:}
  /$5B             {  pop  bx              ;clear flags from stack}
  /$9C             {  pushf                ;save new flags}
  /$72/$08         {  jb NoDiv             ;overflow, no change, Carry set}
  /$C4/$BE/>DVDND  {   les  di,>dvdnd[bp]  ;stuff quotient on long}
  /$AB             {   stosw               ;quotient.lo = ax}
  /$26             {   ES:}
  /$89/$15         {   mov  [di],dx        ;quotient.hi = dx}
                   {NoDiv:}
  /$58             {  pop  ax              ;get flags from stack}
  /$1F             {  pop  DS              ;restore DS}
  /$A2/>FLAGS      {  mov  [<flags],al     ;update global flags}
);
  END;  {of Long_Div}

(* No
  IDIV  no signs
  IMUL
*)


{ INC }

PROCEDURE Long_Inc(VAR long : long_int);
  {INCrement a long int.
   Think of it as L1 := SUCC(L1)
  }
  BEGIN
    Inline(
{;INC1  adding global flags (ZF only)}
  $C4/$BE/>LONG    { les  di,>long[bp] ;long we'll incr}
  /$26             { ES:}
  /$81/$05/$01/$00 { add  word [di],1   ;inc long.lo}
  /$73/$04         { jnc  Exit}
  /$26             { ES:}
  /$FF/$45/$02     { inc  word [di+2]   ;if carry (CF set)}
                   {Exit:}
  /$9F             { lahf               ;get flags reg}
  /$88/$26/>FLAGS  { mov  [<flags],ah   ;set global flags}
);
  END;  {of Long_Inc}

{MOD}

PROCEDURE Long_Mod(VAR dvdnd,dvsr,longmod: long_int);
  {Divide a long integer dividend by a long integer divisor,
   returning the long integer modulus in longmod.
   Like division errors, returns longmod unchanged and Carry set
   if a serious error occurs ( mod 0, dividend < divisor ).
   Think of it as L3 := L1 MOD L2;
  }
  BEGIN
    Inline(
{;MOD4}
  $1E              {  push DS              ;save DS}
  /$FC             {  cld                  ;insure fwd}
  /$31/$C9         {  xor  cx,cx           ;cx=0, clear ZF,CF}
  /$9C             {  pushf                ;save cleared flags on stack}
  /$C5/$B6/>DVSR   {  lds  si,>dvsr[BP]    ;get long divisor's address}
  /$AD             {  lodsw                ;get dvsr.lo, bump si}
  /$89/$C3         {  mov  bx,ax           ;bx=dvsr.lo}
  /$8B/$0C         {  mov  cx,[si]         ;cx=dvsr.hi}
  /$C5/$B6/>DVDND  {  lds  si,>dvdnd[bp]   ;get dividend's address}
  /$AD             {  lodsw                ;ax=dvdnd.lo, bump si}
  /$8B/$14         {  mov  dx,[si]         ;dx=dvdnd.hi}
  /$39/$D1         {  cmp  cx,dx           ;dvsr > dvdnd?}
  /$77/$41         {  ja   JmpStuf         ; yep, no change, CF set}
  /$75/$04         {  jne  NotSame         ;dvsr <> dvdnd}
  /$39/$D8         {   cmp ax,bx           ; lo's same?}
  /$74/$2D         {   je  NoMod0          ; yep, mod 0}
                   {NotSame:}
  /$09/$D2         {  or   dx,dx           ;dvdnd.hi=0?}
  /$75/$04         {  jne  NotZero         ;nope}
  /$09/$C0         {   or  ax,ax           ; dvdnd.lo=0?}
  /$74/$08         {   je  Bad             ; yep, die}
                   {NotZero:}
  /$09/$C9         {  or   cx,cx           ;dvsr.hi=0?}
  /$75/$07         {  jne  NotZero2        ;nope}
  /$09/$DB         {   or  bx,bx           ; dvsr.lo=0?}
  /$75/$03         {   jne NotZero2        ; nope}
                   {Bad:}
  /$F9             {    stc                ;  bad, set CF}
  /$EB/$57         {    jmp short StufMod ;  no change, CF set}
                   {NotZero2:}
  /$09/$C9         {  or  cx,cx            ;dvsr double-word?}
  /$74/$26         {  jz  DoDiv            ; nope, safe to divide}
                   {;double-word division via adds}
  /$31/$F6         {  xor  si,si           ;clear accumulator.lo}
  /$31/$FF         {  xor  di,di           ; & acc.hi}
                   {AddL:}
  /$01/$DE         {  add  si,bx           ;acc.lo}
  /$11/$CF         {  adc  di,cx           ;acc.hi + CF}
  /$72/$11         {  jc   AddCF           ;hi carried, done}
  /$39/$D7         {   cmp di,dx           ; acc > dvdnd?}
  /$72/$F6         {   jb  AddL            ; not yet}
  /$77/$0A         {   ja  AddDone         ; too far, done}
  /$39/$C6         {    cmp si,ax          ;  acc.lo > dvdnd.lo?}
  /$72/$F0         {    jb  AddL           ;  not yet}
  /$77/$04         {    ja  AddDone        ;  too far, done}
                   {NoMod0:}
  /$31/$D2         {     xor dx,dx         ;  remainder=0}
  /$EB/$35         {     jmp short DivDone ; move into ax,done}
                   {AddDone:}
  /$F8             {   clc                 ;insure CF clear}
                   {AddCF:}
  /$19/$CF         {  sbb  di,cx           ;restore acc.hi from last adc}
                   {;  sub  di,cx}
  /$19/$DE         {  sbb  si,bx           ; & acc.lo}
  /$29/$FA         {  sub  dx,di           ;get acc.hi dif}
  /$19/$F0         {  sbb  ax,si           ; & acc.lo dif}
  /$F8             {  clc                  ;insure CF clear}
                   {JmpStuf:}
  /$EB/$2D         {  jmp  short StufMod  ;done}
                   {DoDiv:  ;normal doubleword/word div}
  /$31/$FF         {  xor  di,di           ;get a handy 0}
  /$39/$DA         {  cmp  dx,bx           ;hi >= divisor?}
  /$73/$0C         {  jae  Div32           ;yep, handle special}
  /$09/$D2         {   or  dx,dx           ; dvdnd.hi =0?}
  /$75/$1D         {   jne DivLo           ; nope, do 16-bit div}
  /$39/$D8         {    cmp ax,bx          ;  dvdnd=0 or < divisor?}
  /$77/$19         {    ja  DivLo          ; >, go divide}
  /$74/$19         {    je  DivDone        ;  n/n=1, dx=0, no remainder}
  /$EB/$1B         {    jmp short StufMod ;  n/(>n)=illegal, no change, CF}
                   {Div32:}
  /$89/$C1         {   mov cx,ax           ;save dvdnd.lo}
  /$89/$D0         {   mov ax,dx           ;dvdnd.hi}
  /$31/$D2         {   xor dx,dx           ;cwd, kinda}
  /$F7/$F3         {   div bx              ;divide dvdnd.hi}
  /$91             {   xchg ax,cx          ;cx=quotient.hi,ax=dvdnd.lo}
  /$75/$0A         {   jnz  DivLo          ;more to do, go divide again}
  /$39/$D8         {    cmp ax,bx          ; dvdnd.lo >= divisor?}
  /$77/$06         {    ja  DivLo          ; >, go divide again}
  /$89/$FA         {     mov dx,di ;0      ; assume no remainder}
  /$74/$04         {     je  DivDone       ; =, no remainder}
  /$EB/$06         {     jmp short StufMod ;ax=remainder, dx=0}
                   {DivLo:}
  /$F7/$F3         {  div bx               ;divide}
                   {DivDone: ;remainder in dx}
  /$89/$D0         {  mov ax,dx            ;remainder=mod.lo}
  /$31/$D2         {  xor  dx,dx           ;clear mod.hi}
                   {;ax=quotient.lo, dx=quotient.hi}
                   {StufMod:}
  /$5B             {  pop  bx              ;clear flags from stack}
  /$9C             {  pushf                ;save new flags}
  /$72/$0E         {  jb NoMod             ;overflow, no change, Carry set}
  /$09/$C0         {   or  ax,ax           ; set ZF for mod.lo}
  /$75/$02         {   jne  Stuf1          ; ok, NZ}
  /$09/$D2         {    or dx,dx           ;  ZR, so check mod.hi}
                   {Stuf1:}
  /$C4/$BE/>LONGMOD{   les di,>longmod[bp]  ; get long mod addr}
  /$AB             {   stosw               ; mod.lo = ax}
  /$26             {   ES:}
  /$89/$15         {   mov [di],dx        ; mod.hi = dx}
                   {NoMod:}
  /$58             {  pop  ax              ;get flags from stack}
  /$1F             {  pop  DS              ;restore DS}
  /$A2/>FLAGS      {  mov  [<flags],al     ;update global flags}
);
  END;  {of Long_Mod}


{MUL}

PROCEDURE Int_Mul(mcnd,mplr : INTEGER; VAR product : long_int);
  {Multiply mcnd * mlpr, return a long integer product.
   Global CF always clear, should return global ZF flag correctly.
   }
  BEGIN
    Inline(
{;INTMUL}
  $8B/$86/>MCND    {  mov  ax,>mcnd[bp]    ;get the multiplicand}
  /$8B/$8E/>MPLR   {  mov  cx,>mplr[bp]    ; & multiplier}
  /$F7/$E1         {  mul  cx              ;do the 32-bit mul}
  /$FC             {  cld                  ;insure fwd}
  /$C4/$BE/>PRODUCT{  les  di,>product[bp] ;long product's addr}
  /$AB             {  stosw                ;product.lo}
  /$26             {  ES:}
  /$89/$15         {  mov  [di],dx         ;product.hi}
  /$F8             {  clc                  ;insure carry clear (?)}
  /$9F             {  lahf                 ;flags into ah}
  /$88/$26/>FLAGS  {  mov  [<flags],ah     ;update global flags}
);
  END;  {of Int_Mul}


PROCEDURE Long_Mul_I(VAR mcnd : long_int; mplr : INTEGER);
  {Multiply a long integer multiplicand by an integer multiplier,
   return the long integer mcnd as the product.
   Returns global CF, mcnd unchanged, if long integer overflow.
   Should correctly return Zero flag.
   Zeroes long if long or i are zero.  (Is this right?)
   Think of it as L1 := L1 * i
  }
  BEGIN
    Inline(
{;mul_i5}
  $31/$D2          {  xor  dx,dx         ;insure dx is zeroed}
  /$8B/$86/>MPLR   {  mov  ax,>mplr[bp]  ;get the multiplier}
  /$3D/$01/$00     {  cmp  ax,1          ;1?}
  /$74/$2B         {  je   SetZF         ; n*1=n, just set ZF}
  /$FC             {  cld                ;insure fwd}
  /$C4/$BE/>MCND   {  les  di,>mcnd[bp]  ;get long multiplicand's address}
  /$89/$C1         {  mov  cx,ax         ;cx=mplr}
  /$E3/$1E         {  jcxz StufProd      ;n*0=0: zero product, exit}
  /$26             {  ES:}
  /$8B/$05         {  mov  ax,[di]       ;ax=mcnd.lo}
  /$26             {  ES:}
  /$8B/$55/$02     {  mov  dx,[di+2]     ;dx=mcnd.hi}
  /$09/$C0         {  or  ax,ax}
  /$75/$04         {  jnz  DoMul}
  /$09/$D2         {   or  dx,dx}
  /$74/$15         {   jz  NoMul         ;0*n=0, exit w/ZF}
                   {DoMul:}
  /$89/$C3         {  mov  bx,ax         ;save mcnd.lo}
  /$89/$D0         {  mov  ax,dx         ;ax=mcnd.hi}
  /$31/$D2         {  xor  dx,dx         ;clear dx}
  /$F7/$E1         {  mul  cx            ;mcnd.hi * i,}
                   {;dx=product.hi overflow, ignore}
                   {;ax=product.hi}
  /$93             {  xchg ax,bx         ;ax=mcnd.lo,bx=product.hi}
  /$F7/$E1         {  mul  cx            ;mcnd.lo * i, dx:ax=product}
  /$01/$DA         {  add  dx,bx         ;product.hi+overflow}
  /$72/$06         {  jc   NoMul         ;overflowed, don't change, CF set}
                   {StufProd:}
  /$AB             {  stosw              ;product.lo = ax}
  /$26             {  ES:}
  /$89/$15         {  mov  [di],dx       ;product.hi or 0}
                   {SetZF:}
  /$09/$D0         {  or   ax,dx         ;ZF if lo both 0 (clears CF)}
                   {NoMul:}
  /$9F             {  lahf               ;get flags}
  /$88/$26/>FLAGS  {  mov  [<flags],ah   ;update global flags}
);
  END;  {of Long_Mul_I5}


{MUL or *}

PROCEDURE Long_Mul(VAR mcnd,mplr : long_int);
  {Multiply a long integer multiplicand (mcnd) by a long integer multiplier
   (mplr), return the long integer product in place of mcnd.
   By definition, a true long * a true long (with both hi words nonzero)
   will create an overflow.
   We'll try to trap as many as possible by:
     1:  swapping multiplicand with multiplier if multiplier is larger.
     2:  Leaving a multiplicand ALONE if the MUL overflows.
         (test the Carry flag to see when/if it failed.)
   Zeroes mcnd if mcnd or mplr are zero.  (Is this right?)
   Think of it as L1 := L1 * L2
  }
  BEGIN
    Inline(
{;MUL6}
  $1E              {  push DS            ;save DS}
  /$C5/$B6/>MPLR   {  lds  si,>mplr[BP]  ;long multiplier's address}
  /$FC             {  cld                ;insure fwd}
  /$AD             {  lodsw              ;mlpr.lo, bump si}
  /$8B/$14         {  mov  dx,[si]       ;mlpr.hi}
  /$09/$D2         {  or   dx,dx         ;.hi <> 0?}
  /$75/$07         {  jnz  GetMcnd       ; > 0, multiply}
  /$3D/$01/$00     {   cmp ax,1          ;test mlpr}
  /$74/$4C         {   je  SetZF         ; n*1=n, no change, just set ZF flag}
  /$72/$42         {   jb StufProd       ; n*0=0, product=0, no CF}
                   {GetMcnd:}
  /$89/$C3         {  mov  bx,ax         ;bx=mlpr.lo}
  /$89/$D1         {  mov  cx,dx         ;cx=mlpr.hi}
  /$C5/$B6/>MCND   {  lds  si,>mcnd[BP]  ;long multiplicand's address}
  /$AD             {  lodsw              ;ax=mlcnd.lo}
  /$8B/$14         {  mov  dx,[si]       ;dx=mlcnd.hi}
  /$09/$D2         {  or   dx,dx         ;mlcnd.hi = 0?}
  /$75/$0D         {  jne  ChkSwp        ; nope, continue}
                   {;mcnd.hi is 0}
  /$3D/$01/$00     {   cmp ax,1          ;test mulcnd}
  /$72/$2E         {   jb  StufProd      ; 0*n=0, product=0, no CF}
  /$75/$06         {   jnz ChkSwp        ; not 1, continue}
                   {;mlcnd is 1}
  /$89/$D8         {    mov ax,bx        ;1*n=n, product.lo=mlpr.lo}
  /$89/$CA         {    mov dx,cx        ;product.hi=mlpr.hi}
  /$EB/$26         {    jmp short StufProd ;stuff, exit}
                   {ChkSwp:}
  /$39/$CA         {  cmp  dx,cx         ;mlcnd > mlpr?}
  /$77/$09         {  ja   NoSwap        ;yep, leave it be}
  /$72/$04         {  jb   Swap          ; smaller, swap}
                   {;hi's are the same}
  /$39/$D8         {   cmp  ax,bx        ;mlcnd > mlpr?}
  /$73/$03         {   jae  NoSwap       ;ok, >=, leave it be}
                   {Swap:}
  /$93             {   xchg ax,bx        ;swap lo's}
  /$87/$CA         {   xchg dx,cx        ; & hi's}
                   {NoSwap:}
  /$31/$FF         {  xor  di,di         ;get handy 0}
  /$39/$F9         {  cmp  cx,di ;0      ;mlpr.hi > 0? (sets CF)}
  /$72/$1D         {  jc   NoMul         ;overflow, exit no change, CF set}
  /$89/$C1         {  mov  cx,ax         ;save mlcnd.lo}
  /$89/$D0         {  mov  ax,dx         ;ax=mlcnd.hi}
  /$31/$D2         {  xor  dx,dx         ;clear dx}
  /$F7/$E3         {  mul  bx            ;mlcnd.hi * mlpr.lo,}
  /$39/$FA         {  cmp  dx,di ;0      ;any overflow? (sets CF)}
  /$72/$11         {  jc   NoMul         ;overflow, exit no change, CF set}
                   {;dx=0, ax=product.hi}
  /$91             {  xchg ax,cx         ;ax=mlcnd.lo,cx=product.hi}
  /$F7/$E3         {  mul  bx            ;mlcnd.lo * mlpr.lo,}
                   {                     ; product.lo in dx:ax}
  /$01/$CA         {  add  dx,cx         ;product.hi+overflow}
  /$72/$0A         {  jc   NoMul         ;bad, exit no change, CF set}
                   {StufProd:            ;enter here with CF, ZF you want}
  /$C4/$BE/>MCND   { les  di,>mcnd[BP]   ;long mlcnd's address}
  /$AB             { stosw               ;stuff product.lo}
  /$26             { ES:}
  /$89/$15         { mov  [di],dx        ;stuff product.hi}
                   {SetZF:}
  /$09/$D0         { or  ax,dx           ;Set ZF for product (clears CF)}
                   {NoMul:}
  /$1F             {  pop  DS            ;restore DS}
  /$9F             {  lahf               ;get flags}
  /$88/$26/>FLAGS  {  mov  [<flags],ah   ;set global flags}
);
  END;  {of Long_Mul6}


{ NEG }  {although who the HELL would wanna do this....}

PROCEDURE Long_Neg(VAR long : long_int);
  {NEGate a long int.  I assume you want to call the high bit
   of the high word the sign bit, and flip it.
   Don't know HOW to explain this in Pascal!
   What's < 0?  And how big an integer do you add to make it positive?
  }
  BEGIN
    Inline(
  $C4/$BE/>LONG    {les  di,>long[bp]  ;long int's vector}
  /$26             {ES:}
  /$F7/$5D/$02     {neg  word [di+2]}
);
  END;  {of Long_Neg}


{ NOT }

PROCEDURE Long_Not(VAR long, notted : long_int);
  {NOT a long int, returning a new long int.
   Think of it as L2 := NOT L1
   Silly, but I use this extensively in certain records to
   do a brute-force backup/check of critical values.  I stuff
   in the long, and its NOT right next to it.  If they differ
   (by doing a Long_Not_Test .. see below), I know ONE of them
   got hit and can suspect the data.
  }
  BEGIN
    Inline(
  $8C/$DB          { mov  bx,DS          ;preserve DS}
  /$C5/$B6/>LONG   { lds  si,>long[bp]   ;the source long int vector}
  /$C4/$BE/>NOTTED { les  di,>notted[bp] ;our target long int}
  /$FC             { cld        ;insure fwd}
  /$AD             { lodsw      ;long.lo}
  /$F7/$D0         { not  ax    ;not it}
  /$AB             { stosw      ;stuff in notted.lo}
  /$AD             { lodsw      ;long.hi}
  /$F7/$D0         { not  ax    ;not it}
  /$AB             { stosw      ;stuff in notted.hi}
  /$8E/$DB         { mov  DS,bx ;restore DS}
);
  END;  {of long_Not}


{ NOT }

FUNCTION Long_Not_Test(VAR long, longNot : long_int ) : BOOLEAN;
  {Returns TRUE if NOT(long) equals longNot.
   Silly, I know, but I use this extensively when testing
   a long against its sister NOT.  See the procedure Long_Not.
  }
  BEGIN
    Inline(
  $8C/$DB          { mov  bx,DS     ;preserve DS}
  /$31/$D2         { xor  dx,dx     ;True flag}
  /$C5/$B6/>LONG   { lds  si,>long[bp]   ;the source long int vector}
  /$C4/$BE/>LONGNOT{ les  di,>longNot[bp]  ;our test long int vector}
  /$FC             { cld            ;insure fwd}
  /$AD             { lodsw          ;long.lo}
  /$F7/$D0         { not  ax        ;NOT it}
  /$26             { ES:}
  /$39/$05         { cmp  [di],ax   ;same as longNot.lo?}
  /$75/$0A         { jne  False     ; nope, failed}
  /$AD             { lodsw          ;long.hi}
  /$F7/$D0         { not  ax        ;NOT it}
  /$26             { ES:}
  /$39/$45/$02     { cmp  [di+2],ax ;same as longNot.hi?}
  /$75/$01         { jne  False     ; nope, failed}
  /$42             {  inc dx        ;True flag}
                   {False:}
(*
Just for your edification, and in case you're wondering how to return}
  a boolean, the stack looks like this:}
  0,1 : return address}
  2,3 : the saved sp}
  4,5 : long ip}
  6,7 : long seg}
  8,9 : longNot ip}
  A,B : longNot seg}
  C   : the boolean return value}
*)
  /$88/$56/$0C     {  mov  [bp+$0C],dl    ;hand Turbo the boolean}
  /$8E/$DB         {  mov  DS,bx          ;restore DS}
);
  END;  {of Long_Not_Test}


{ OR }

PROCEDURE Long_Or(VAR long, orrer : long_int);
  {OR the first long int with a second long int,
   returning the changed first long int.
   Think of it as L1 := L1 OR L2
  }
  BEGIN
    Inline(
{;OR1  adding global flags}
  $8C/$DB          { mov  bx,DS          ;preserve DS}
  /$FC             { cld                 ;insure fwd}
  /$C5/$B6/>ORRER  { lds  si,>orrer[bp]  ;the orrer long int vector}
  /$C4/$BE/>LONG   { les  di,>long[bp]   ;gonna stuff long}
  /$AD             { lodsw               ;orrer.lo}
  /$26             { ES:}
  /$09/$05         { or   [di],ax        ;orrer.lo OR long.lo}
  /$9F             { lahf                ;get flag reg}
  /$88/$E6         { mov  dh,ah          ;save flag reg a sec}
  /$AD             { lodsw               ;orrer.hi}
  /$26             { ES:}
  /$09/$45/$02     { or   [di+2],ax      ;long.hi OR orrer.hi}
  /$9F             { lahf                ;get new flags}
  /$08/$F4         { or   ah,dh          ;OR with old flags}
  /$8E/$DB         { mov  DS,bx          ;restore DS}
  /$88/$26/>FLAGS  { mov  [<flags],ah    ;update global flags}
);
  END;  {of Long_Or}


{ RCL }

PROCEDURE Long_RcL(VAR long : long_int; rclv : INTEGER);
  { Rotate an unsigned long integer left through the Carry Flag,
    1..31 bits, return the long integer.
    Think of it as L1 := L1 RcL rclv
    This requires a global "flags" integer to remember the carry flag
    (and others) from call to call.
    You can "CLC" (clear carry flag) by clearing the "flags" 0 bit,
         or "STC" (set carry flag) by setting the "flags" 0 bit.
  }
  BEGIN
    Inline(
{;RCL1  Adding global flags}
  $31/$DB          {  xor  bx,bx          ;insure bl=0}
  /$8B/$86/>RCLV   {  mov  ax,>rclv[bp]   ;rcl parm (1..31?)}
  /$25/$1F/$00     {  and  ax,31          ;mask to legal maximum}
  /$89/$C1         {  mov  cx,ax          ;needed in cx}
  /$E3/$21         {  jcxz NoRcL          ;0, ignore this fool}
  /$C4/$BE/>LONG   {  les  di,>long[bp]   ;long we'll RCL}
  /$26             {  ES:}
  /$8B/$05         {  mov  ax,[di]        ;long.lo}
  /$26             {  ES:}
  /$8B/$55/$02     {  mov  dx,[di+2]      ;long.hi}
  /$8A/$1E/>FLAGS  {  mov  bl,[<Flags]    ;global flags}
  /$80/$E3/$01     {  and  bl,1           ;clear all but carry bit}
  /$D0/$EB         {  shr  bl,1           ;set CF appropriately}
                   {RcLLup:}
  /$D1/$D0         {  rcl  ax,1           ;rcl .lo, CF > bit 0, bit 7 > CF}
  /$D1/$D2         {  rcl  dx,1           ;rcl .hi, CF > bit 0, bit 7 > CF}
  /$E2/$FA         {  loop RcLLup         ;do 1..31}
                   {;exiting, any hi word carry is now in CF.}
  /$D0/$D3         {  rcl  bl,1           ;pick up the carry bit}
  /$FC             {  cld                 ;insure fwd}
  /$AB             {  stosw               ;stuff long.lo}
  /$26             {  ES:}
  /$89/$15         {  mov [di],dx         ;stuff long.hi}
                   {NoRcL:}
  /$88/$1E/>FLAGS  {  mov  [<flags],bl    ;save new flags}
);
  END;  {of Long_RcL}


{ RCR }

PROCEDURE Long_RcR(VAR long : long_int; rcrv : INTEGER);
  { Rotate an unsigned long integer right through the Carry Flag,
    1..31 bits, return the long integer.
    Think of it as L1 := L1 RcR rcrv
    This requires a global "flag" integer to remember the carry flag
    (plus other flags) from call to call.
    You can "CLC" (clear carry flag) by clearing the "flags" 0 bit,
         or "STC" (set carry flag) by setting the "flags" 0 bit.
  }
  BEGIN
(*
We first mask and SHR the global integer "flags"
to initialize the carry flag (CF) from the last time we did this.
We then RCR the high word, moving the previous CF into .hi's bit 7,
and .hi's bit 0 into CF.
We then RCR the low word, moving the previous CF into .lo's bit 7,
and .lo's bit 0 into CF, thus producing the desired 32-bit RCR effect.
The LOOP doesn't affect any flags.
When we fall through the loop, we set the global "flags" according
to CF for next time.
We also do a bit of strangeness to insure .hi's bit 7 is set
appropriately per CF.
*)
    Inline(
{;RCR1  adding global flags}
  $31/$DB          {  xor  bx,bx          ;insure bl=0}
  /$8B/$86/>RCRV   {  mov  ax,>rcrv[bp]   ;rcr parm (1..31?)}
  /$25/$1F/$00     {  and  ax,31          ;mask to legal max}
  /$89/$C1         {  mov  cx,ax          ;need in cx}
  /$E3/$21         {  jcxz NoRcR          ;ignore this fool}

  /$C4/$BE/>LONG   {  les  di,>long[bp]   ;long we'll RCR}
  /$26             {  ES:}
  /$8B/$05         {  mov  ax,[di]        ;long.lo}
  /$26             {  ES:                 ; don't disturb di}
  /$8B/$55/$02     {  mov  dx,[di+2]      ;long.hi}

  /$8A/$1E/>FLAGS  {  mov  bl,[<flags]    ;global flags}
  /$80/$E3/$01     {  and  bl,1   ;mask all but carry bit}
  /$D0/$EB         {  shr  bl,1   ;set CF accordingly}
                   {RcRLup:}
  /$D1/$DA         {  rcr  dx,1   ;rcr .hi: CF > bit 7, bit 0 > CF}
  /$D1/$D8         {  rcr  ax,1   ;rcr .lo, CF > bit 7, bit 0 > CF}
  /$E2/$FA         {  loop RcRLup ;do 1..31}
  /$D0/$D3         {  rcl  bl,1   ;CF into bl's "carry bit"}
  /$FC             {  cld         ;insure fwd}
  /$AB             {  stosw       ;stuff long.lo}
  /$26             {  ES:}
  /$89/$15         {  mov  [di],dx ;stuff long.hi}
                   {NoRcR:}
  /$88/$1E/>FLAGS  {  mov  [<flags],bl    ;update global flags}
);
  END;  {of Long_RcR}


{ ROL }

PROCEDURE Long_RoL(VAR long : long_int; rolv : INTEGER);
  { Rotate an unsigned long integer left, moving hi bit
    into the low bit, 1..31 bits, return the long integer.
    Think of it as L1 := L1 RoL rolv
  }
  BEGIN
(*
We enter the loop with CF clear.
We then RCL .lo, moving .lo's bit 7 into CF.
We then RCL .hi, moving the previous CF
into .hi's bit 0, and .hi's bit 7 into CF.
The CF moves back into .lo's bit 0 next loop,
thus producing the desired 32-bit ROL effect.
The LOOP doesn't affect any flags.
When we fall through the loop, we use the CF
to make sure .lo's bit 0 is set per CF.
*)
    Inline(
{;ROL1  adding global flags}
  $31/$DB          {  xor  bx,bx          ;insure bl=0}
  /$8B/$86/>ROLV   {  mov  ax,>rolv[bp]   ;rol parm (1..31)}
  /$25/$1F/$00     {  and  ax,31          ;mask to legal maximum}
  /$89/$C1         {  mov  cx,ax          ;need in cx}
  /$E3/$1B         {  jcxz NoRol          ;ignore this fool}

  /$C4/$BE/>LONG   {  les  di,>long[bp]   ;long we'll RCL}
  /$26             {  ES:}
  /$8B/$05         {  mov  ax,[di]        ;long.lo}
  /$26             {  ES:}
  /$8B/$55/$02     {  mov  dx,[di+2]      ;long.hi}
  /$F8             {  clc}
                   {RolLup:}
  /$D1/$D0         {  rcl  ax,1           ;rcl .lo, CF > bit 0, bit 7 > CF}
  /$D1/$D2         {  rcl  dx,1           ;rcl .hi, CF > bit 0, bit 7 > CF}
  /$E2/$FA         {  loop RolLup         ;do 1..31}
                   {;exiting, any hi word carry is now in CF.}
  /$11/$C8         {  adc  ax,cx          ;in case .hi had a carry}
  /$D0/$D3         {  rcl  bl,1           ;pick up CF in case .lo carried}
  /$FC             {  cld                 ;insure fwd}
  /$AB             {  stosw               ;stuff long.lo}
  /$26             {  ES:}
  /$89/$15         {  mov  [di],dx        ;stuff long.hi}
                   {NoRoL:}
  /$88/$1E/>FLAGS  {  mov  [<flags],bl    ;update global flags}
);
  END;  {of Long_RoL}

{ ROR }

PROCEDURE Long_RoR(VAR long : long_int; rorv : INTEGER);
  { Rotate an unsigned long integer right, moving lo bit
    into the hi bit, 1..31 bits, return the long integer.
    Think of it as L1 := L1 RoR rorv
  }
  BEGIN
(*
We enter the loop with CF clear.
We then RCR .hi, moving .hi's bit 0 into CF.
We then RCR .lo, moving the previous CF
into .lo's bit 7, and .lo's bit 0 into CF.
The CF moves back into .hi's bit 7 next loop,
thus producing the desired 32-bit ROR effect.
The LOOP doesn't affect any flags.
When we fall through the loop, we use the CF
to make sure .hi's bit 7 is set per CF.
*)
    Inline(
{;ROR1  adding global flags}
  $31/$DB          {  xor  bx,bx          ;insure bl=0}
  /$8B/$86/>RORV   {  mov  ax,>rorv[bp]   ;ror parm (1..31)}
  /$25/$1F/$00     {  and  ax,31          ;mask to legal max}
  /$89/$C1         {  mov  cx,ax          ;needed in cx}
  /$E3/$1E         {  jcxz NoRoR          ;ignore this fool}

  /$C4/$BE/>LONG   {  les  di,>long[bp]   ;long we'll RCR}
  /$26             {  ES:}
  /$8B/$05         {  mov  ax,[di]        ;long.lo}
  /$26             {  ES:}
  /$8B/$55/$02     {  mov  dx,[di+2]      ;long.hi}
  /$F8             {  clc}
                   {RorLup:}
  /$D1/$DA         {  rcr  dx,1   ;rcr .hi: CF > bit 7, bit 0 > CF}
  /$D1/$D8         {  rcr  ax,1   ;rcr .lo, CF > bit 7, bit 0 > CF}
  /$E2/$FA         {  loop RorLup ;do 1..31}
  /$73/$05         {  jnc  S1     ;no carry, skip this mess}
  /$D0/$D3         {   rcl bl,1   ;pick up the carry bit}
  /$80/$CE/$80     {   or  dh,$80 ;mask hi bit}
                   {S1:}
  /$FC             {  cld         ;insure fwd}
  /$AB             {  stosw       ;stuff long.lo}
  /$26             {  ES:}
  /$89/$15         {  mov  [di],dx ;stuff long.hi}
                   {NoRoR:}
  /$88/$1E/>FLAGS  {  mov  [<flags],bl ;update global carry}
);
  END;  {of Long_RoR}


{ SHL }

PROCEDURE Long_ShL(VAR long : long_int; shft : INTEGER);
  { Shift an unsigned long integer left 1..32 bits,
    return the long integer.
    Think of it as L1 := L1 ShL shft
  }
  BEGIN
    Inline(
{;SHL1  adding global flags}
  $8B/$86/>SHFT    {  mov  ax,>shft[bp] ;shift parm (1..32?)}
  /$25/$1F/$00     {  and  ax,31        ;mask to legal values}
  /$08/$E4         {  or   ah,ah        ;clear CF,ZF}
  /$9C             {  pushf             ;save flags on stack}
  /$89/$C1         {  mov  cx,ax        ;need in cx}
  /$E3/$1A         {  jcxz NoShL        ;it was zero, ignore}

  /$8C/$DB         {  mov  bx,DS        ;preserve DS}
  /$C5/$B6/>LONG   {  lds  si,>long[bp] ;long we'll shift left}
  /$89/$F7         {  mov  di,si        ;for stuffing back}
  /$FC             {  cld               ;insure fwd}
  /$AD             {  lodsw             ;snarf low word}
  /$8B/$14         {  mov  dx,[si]      ;get hi word}

{;Using RCL shifts the hi word left, moving}
{;any carry (CF) from lo word shift into}
{;hi word's bit 0.}
                   {ShLLup:}
  /$D1/$E0         {  shl  ax,1     ;shift lo word}
  /$D1/$D2         {  rcl  dx,1     ;shift hi word, CF > bit 0}
  /$E2/$FA         {  loop ShLLup   ;do 1..32 (I suppose)}
  /$59             {  pop  cx       ;clear flags from stack}
  /$9C             {  pushf         ;save new flags on stack}
  /$89/$05         {  mov  [di],ax  ;stuff lo word}
  /$89/$14         {  mov  [si],dx  ;stuff hi word}
  /$8E/$DB         {  mov  DS,bx    ;restore DS}
                   {NoShL:}
  /$8F/$06/>FLAGS  {  pop  [>flags] ;update global flags}
);
  END;  {of Long_ShL}


{ SHR }

PROCEDURE Long_ShR(VAR long : long_int; shft : INTEGER);
  { Shift an unsigned long integer right 1..32 bits,
    return the long integer.
    Think of it as L1 := L1 ShR shft
  }
  BEGIN
    Inline(
{;SHR1  adding global flags}
  $8B/$86/>SHFT    {  mov  ax,>shft[bp]  ;shift parm (1..32?)}
  /$25/$1F/$00     {  and  ax,31         ;max legal shift}
  /$08/$E4         {  or   ah,ah         ;clear CF,ZF}
  /$9C             {  pushf              ;save flags on stack}
  /$89/$C1         {  mov  cx,ax         ;needed in cx}
  /$E3/$1A         {  jcxz NoShR         ;0, ignore this fool}
  /$8C/$DB         {  mov  bx,DS         ;preserve DS}
  /$C5/$B6/>LONG   {  lds  si,>long[bp]  ;long we'll shift right}
  /$89/$F7         {  mov  di,si         ;for stuffing back}
  /$FC             {  cld                ;insure fwd}
  /$AD             {  lodsw              ;snarf low word}
  /$8B/$14         {  mov  dx,[si]       ;get hi word}

{;Use RCR instr to shift lo word right.}
{;A hi word carry will move carry flag}
{;(CF) into lo word's hi bit.}

                   {ShRLup:}
  /$D1/$EA         {  shr  dx,1    ;shift hi word}
  /$D1/$D8         {  rcr  ax,1    ;shift lo word, CF > bit 7}
  /$E2/$FA         {  loop ShRLup  ;do 1..32?}
  /$59             {  pop  cx      ;clear flags from stack}
  /$9C             {  pushf        ;save new flags on stack}
  /$89/$05         {  mov  [di],ax ;stuff lo word}
  /$89/$14         {  mov  [si],dx ;stuff hi word}
  /$8E/$DB         {  mov  DS,bx   ;restore DS}
                   {NoShR:}
  /$8F/$06/>FLAGS  {  pop  [>flags] ;update global flags}
);
  END;  {of Long_ShR}


{ SUB }

PROCEDURE Long_Sub(VAR long, sub : long_int);
  { Subtracts two long integers, returning new long.
   Think of it as L1 := L1 - L2
  }
  BEGIN
    Inline(
{;SUB1  adding global flags}
  $8C/$DB          { mov  bx,DS}
  /$C5/$B6/>SUB    { lds  si,>sub[bp]  ;long we'll subtract}
  /$FC             { cld               ;insure forward}
  /$AD             { lodsw             ;snarf sub.lo}
  /$8B/$0C         { mov  cx,[si]      ;snarf sub.hi}
  /$09/$C0         { or   ax,ax        ;zero?}
  /$75/$02         { jne  Sub1         ; nope, continue}
  /$E3/$0B         {  jcxz NoSub       ; also 0, forget it}
                   {Sub1:}
  /$C4/$BE/>LONG   { les  di,>long[bp] ;long we'll subtract from}
  /$26             { ES:}
  /$29/$05         { sub  [di],ax      ;sub from long.lo}
  /$26             { ES:}
  /$19/$4D/$02     { sbb  [di+2],cx    ;sub from long.hi with borrow}
                   {NoSub:}
  /$8E/$DB         { mov  DS,bx        ;restore}
  /$9F             { lahf              ;move flags into AH}
  /$88/$26/>FLAGS  { mov  [<flags],ah  ;update global flags}
);
  END;  { of Long_Sub}


{ SUB }

PROCEDURE Long_Sub_I(VAR long : long_int; i : INTEGER);
  { Subtract an unsigned integer from an unsigned long integer,
    return the long integer.
    Think of it as L1 := L1 - i
  }
  BEGIN
    Inline(
{;SUB_I1  adding global flags}
  $C4/$BE/>LONG    { les  di,>long[bp] ;long we'll subtract}
  /$8B/$86/>I      { mov  ax,>i[bp]    ;integer parm}
  /$26             { ES:}
  /$29/$05         { sub  [di],ax      ;sub long.lo}
  /$73/$06         { jnb  Exit}
  /$26             { ES:}
  /$81/$6D/$02
  /$01/$00         { sub  word [di+2],1 ;if borrow}
                   {Exit:}
  /$9F             { lahf               ;get flags}
  /$88/$26/>FLAGS  { mov  [<flags],ah   ;update global flags}
);
  END;  {of Long_Sub_I}


{ XOR }

PROCEDURE Long_Xor(VAR long, xorval : long_int);
  {XOR the first long int with a second long int,
   returning the changed first long int.
   Think of it as L1 := L1 XOR L2
  }
  BEGIN
    Inline(
  $8C/$DB          { mov  bx,DS          ;preserve DS}
  /$FC             { cld                 ;insure fwd}
  /$C5/$B6/>XORVAL { lds  si,>xorval[bp] ;the xorval long int vector}
  /$C4/$BE/>LONG   { les  di,>long[bp]   ;gonna stuff long}
  /$AD             { lodsw               ;xorval.lo}
  /$26             { ES:}
  /$31/$05         { xor [di],ax         ;xorval.lo XOR long.lo}
  /$AD             { lodsw               ;xorval.hi}
  /$26             { ES:}
  /$31/$45/$02     { xor [di+2],ax       ;long.hi XOR xorval.hi}
  /$8E/$DB         { mov  DS,bx          ;restore DS}
);
  END;  {of Long_Xor}
