/****************************************************************************
*
*								 Zen Timer
*
*							   From the book
*						 "Zen of Assembly Language"
*							Volume 1, Knowledge
*
*							 by Michael Abrash
*
*					Simple Test program by Kendall Bennett
*
* Filename:		$RCSfile: main.c $
* Version:		$Revision: 1.1 $
*
* Language:		ANSI C
* Environment:	MS DOS (IBM PC)
*
* Description:	Test program for the Zen Timer Library.
*
* $Id: main.c 1.1 92/01/27 21:43:06 kjb release $
*
* Revision History:
* -----------------
*
* $Log:	main.c $
* Revision 1.1  92/01/27  21:43:06  kjb
* Initial revision
* 
****************************************************************************/

#include <stdio.h>
#include "debug.h"
#include "ztimer.h"

/*------------------------- Global variables ------------------------------*/

char	*rcsid = "$Id: main.c 1.1 92/01/27 21:43:06 kjb release $";

/*-------------------------- Implementation -------------------------------*/

/* The following routine takes a long count in microseconds and outputs
 * a string representing the count in seconds. It could be modified to
 * return a pointer to a static string representing the count rather
 * than printing it out.
 */

void ReportTime(ulong count)
{
	ulong	secs;

	secs = count / 1000000L;
	count = count - secs * 1000000L;
	printf("Time taken: %lu.%06lu seconds\n",secs,count);
}

int		i,j;								/* NON register variables! */

int main(void)
{
	ulong	count;

	/* Test the precision timer routine */

	PZTimerOn();
	for (i = 0; i < 10000; i++)
		i = i;
	PZTimerOff();
	PZTimerReport();
	count = PZTimerCount();
	printf("Count returned: %lu\n",count);

	/* Test the precision timer routine for overflow */

	PZTimerOn();
	for (j = 0; j < 10; j++)
		for (i = 0; i < 20000; i++)
			i = i;
	PZTimerOff();
	PZTimerReport();
	count = PZTimerCount();
	printf("Count returned: %lu\n",count);

	/* Test the long period Zen Timer (we don't check for overflow coz
	 * it would take tooooo long!)
	 */

	LZTimerOn();
	for (j = 0; j < 10; j++)
		for (i = 0; i < 20000; i++)
			i = i;
	LZTimerOff();
	LZTimerReport();
	ReportTime(LZTimerCount());

	return 0;
}
