/****************************************************************************
*
*						   The Zen Timer Library
*
*							   From the book
*						 "Zen of Assembly Language"
*							Volume 1, Knowledge
*
*							 by Michael Abrash
*
*					  Modifications by Kendall Bennett
*
* Filename:		$RCSfile: ztimer.h $
* Version:		$Revision: 1.5 $
*
* Language:		ANSI C
* Environment:	IBM PC (MS DOS)
*
* Description:	Header file for the Zen Timer library. Provides a number
*				of routines to accurately time segments of code. A
*				precision timer is provided for highly accurate timing of
*				code that takes less than 54 ms to execute, and a long
*				period timer is provided to time code that takes up to
*				one hour to execute. All output is in microseconds.
*
* $Id: ztimer.h 1.5 92/01/27 21:40:01 kjb release $
*
* Revision History:
* -----------------
*
* $Log:	ztimer.h $
* Revision 1.5  92/01/27  21:40:01  kjb
* Converted to a memory model independant library, and released to the
* public.,
* 
* Revision 1.4  91/12/31  19:34:51  kjb
* Changed include file directories.
* 
* Revision 1.3  91/12/26  17:56:38  kjb
* Added dependency on DEBUG.H
* 
* Revision 1.2  91/11/16  17:12:20  kjb
* Modified to return a long integer representing the count rather than a 
* string.
* 
* Revision 1.1  91/11/14  17:19:58  kjb
* Initial revision
* 
****************************************************************************/

#ifndef	__ZTIMER_H
#define	__ZTIMER_H

#ifndef __DEBUG_H
#include "debug.h"
#endif

/*-------------------------- Function Prototypes --------------------------*/

#ifdef	__cplusplus
extern "C" {			/* Use "C" linkage when in C++ mode	*/
#endif

/* Precision timing routines in PZTIMER.ASM */

void	far PZTimerOn(void);
void	far PZTimerOff(void);
void	far PZTimerReport(void);
ulong	far PZTimerCount(void);

/* Long period timing routines in LZTIMER.ASM */

void 	far LZTimerOn(void);
void 	far LZTimerOff(void);
void 	far LZTimerReport(void);
ulong 	far LZTimerCount(void);

#ifdef	__cplusplus
}						/* End of "C" linkage for C++	*/
#endif

#endif	__ZTIMER_H
